@@ -63,6 +63,7 @@ $status=0;
 $fptr=CFITSIO::open_file('tq123x.kjl',READWRITE,$status);
 printf "  ffopen fptr, status  = %d %d (expect an error)\n",$fptr,$status;
 eval {
+	$status = 115; # cheat!!!
 	$fptr->close_file($status);
 };
 printf "  ffclos status = %d\n\n", $status;
@@ -151,13 +152,8 @@ $fptr->write_key_fixcmp('key_pkfc',$onekey,6,'fxpkfc comment',$status)
 $fptr->write_key_fixdblcmp('key_pkfm',$ondkey,14,'fxpkfm comment',$status)
 	and print "ffpkfm status = $status\n";
 
-##############################################
-# PROBLEM: erratic Bus Errors on Solaris 2.5 #
-##############################################
-$comment='key_pkls';
 $fptr->write_key_longstr(
-	$comment,
-	#'key_pkls',
+	'key_pkls',
 	'This is a very long string value that is continued over more than one keyword.',
 	'fxpkls comment',
 	$status,
@@ -370,9 +366,6 @@ $anynull or print "ERROR: ffgpf_ did not detect null values\n";
 #  close and reopen file multiple times  #
 ##########################################
 
-   ############################################
-   # PROBLEM: SEGV on Solaris 2.5, Perl 5.004 #
-   ############################################
 for ($ii=0;$ii<10;$ii++) {
 	$fptr->close_file($status) and
 		print("ERROR in ftclos (1) = $status"), goto ERRSTATUS;
@@ -582,9 +575,6 @@ $fptr->insert_key_lng('KY_IKYJ',49,"ikyj comment", $status);
 $fptr->insert_key_log('KY_IKYL',1, "ikyl comment", $status);
 $fptr->insert_key_flt('KY_IKYE',12.3456, 4, "ikye comment", $status);
 $fptr->insert_key_dbl('KY_IKYD',12.345678901234567, 14, "ikyd comment", $status);
-   ############################################
-   # PROBLEM: SEGV on Solaris 2.5, Perl 5.004 #
-   ############################################
 $fptr->insert_key_fixflt('KY_IKYF',12.3456, 4, "ikyf comment", $status);
 $fptr->insert_key_fixdbl('KY_IKYG',12.345678901234567, 13, "ikyg comment", $status);
 
@@ -1121,13 +1111,8 @@ $fptr->read_colnull_sht(5,1,1,$nrows,$iinarray,$larray2,$anynull,$status);
 $fptr->read_colnull_int(6,1,1,$nrows,$kinarray,$larray2,$anynull,$status);
 $fptr->read_colnull_flt(7,1,1,$nrows,$einarray,$larray2,$anynull,$status);
 $fptr->read_colnull_dbl(8,1,1,$nrows,$dinarray,$larray2,$anynull,$status);
-   ####################################
-   # PROBLEM: sporadic SEGVs on Linux #
-   ####################################
-#$fptr->read_colnull_cmp(9,1,1,$nrows,$cinarray,$larray2,$anynull,$status);
-#$fptr->read_colnull_dblcmp(10,1,1,$nrows,$minarray,$larray2,$anynull,$status);
-$fptr->read_col_cmp(9,1,1,$nrows,98.,$cinarray,$anynull,$status);
-$fptr->read_col_dblcmp(10,1,1,$nrows,98.,$minarray,$anynull,$status);
+$fptr->read_colnull_cmp(9,1,1,$nrows,$cinarray,$larray2,$anynull,$status);
+$fptr->read_colnull_dblcmp(10,1,1,$nrows,$minarray,$larray2,$anynull,$status);
 
 print "\nRead columns with ffgcf_:\n";
 for ($ii=0;$ii<10;$ii++) {
@@ -1751,11 +1736,7 @@ $fptr->read_col(TSHORT,2,1,1,10,$inul,$iinarray,$anynull,$status);
 $fptr->read_col(TINT,3,1,1,10,$knul,$kinarray,$anynull,$status);
 $fptr->read_col(TLONG,3,1,1,10,$jnul,$jinarray,$anynull,$status);
 $fptr->read_col(TFLOAT,4,1,1,10,$enul,$einarray,$anynull,$status);
-   #######################################################
-   # PROBLEM: Bus Error on Solaris 2.5 (TFLOAT fixes it) #
-   #######################################################
-#$fptr->read_col(TDOUBLE,5,1,1,10,$dnul,$dinarray,$anynull,$status);
-$fptr->read_col_dbl(5,1,1,10,$dnul,$dinarray,$anynull,$status);
+$fptr->read_col(TDOUBLE,5,1,1,10,$dnul,$dinarray,$anynull,$status);
 
 print "\nColumn values written with ffpcl and read with ffgcl:\n";
 $npixels = 10;
